package org.objectweb.jonathan.libs.kernel.decoders;

import junit.framework.*;

//import org.objectweb.jonathan.apis.kernel.Kernel;
import org.objectweb.jonathan.libs.kernel.Kernel;
import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Element;
import org.objectweb.jonathan.apis.kernel.ClassPathFinder;
import org.objectweb.jonathan.apis.kernel.decoders.File2Configuration;
// import org.objectweb.jonathan.apis.resources.Scheduler;

import org.objectweb.jonathan.libs.kernel.JConfigurationFactory;

import java.util.Enumeration;


public class OpenXMLDecoderTest extends TestCase {

   public OpenXMLDecoderTest(String name) {
      super(name);
   }

	public static void main(String args[]) {
		junit.textui.TestRunner.run(OpenXMLDecoderTest.class);
	}
   
   public void testImportXML() {
      try {
         
         Context $initial_context = Kernel.newConfiguration(OpenXMLDecoderTest.class);
         ClassPathFinder $finder = (ClassPathFinder) $initial_context.getValue("classpath_finder",(char) 0);
         File2Configuration $decoder = (File2Configuration) 
            $initial_context.getValue("jonathan/decoders/OpenXMLDecoder/instance",'/');
         
         JConfigurationFactory $context_factory = new JConfigurationFactory();

         Context $context = $context_factory.newContext(null,$finder);
         
         $decoder.importElement($context,
                                "org/objectweb/jonathan/libs/kernel/decoders/test_import_xml.xml",
                                $context_factory,
                                $finder);

         Character $jscheduler_max_waiting_char = (Character)
            $context.getValue("/JScheduler/max_waiting_char",'/');
         assertEquals(new Character('a'),$jscheduler_max_waiting_char);

         Byte $jscheduler_max_waiting_byte = (Byte)
            $context.getValue("/JScheduler/max_waiting_byte",'/');
         assertEquals(new Byte((byte)50),$jscheduler_max_waiting_byte);

         Short $jscheduler_max_waiting_short = (Short)
            $context.getValue("/JScheduler/max_waiting_short",'/');
         assertEquals(new Short((short)5000),$jscheduler_max_waiting_short);

         Integer $jscheduler_max_waiting_int = (Integer)
            $context.getValue("/JScheduler/max_waiting_int",'/');
         assertEquals(new Integer(500000000),$jscheduler_max_waiting_int);

         Long $jscheduler_max_waiting_long = (Long)
            $context.getValue("/JScheduler/max_waiting_long",'/');
         assertEquals(new Long(500000000),$jscheduler_max_waiting_long);

         Float $jscheduler_max_waiting_float = (Float)
            $context.getValue("/JScheduler/max_waiting_float",'/');
         assertEquals(new Float(500.5),$jscheduler_max_waiting_float);

         Double $jscheduler_max_waiting_double = (Double)
            $context.getValue("/JScheduler/max_waiting_double",'/');
         assertEquals(new Double(5.5E10),$jscheduler_max_waiting_double);


         Integer $toto_max_waiting = (Integer)
            $context.getValue("/toto/max_waiting",'/');
         assertEquals(new Integer(34),$toto_max_waiting);

         Boolean $jscheduler_verbose = (Boolean)
            $context.getValue("/JScheduler/verbose",'/');
         assertEquals(Boolean.FALSE,$jscheduler_verbose);

         Boolean $toto_verbose = (Boolean)
            $context.getValue("/toto/verbose",'/');
         assertSame($toto_verbose,$jscheduler_verbose);

         Integer $jscheduler_dummy = (Integer)
            $context.getValue("/JScheduler/dummy",'/');
         assertEquals(new Integer(13),$jscheduler_dummy);

         Integer $toto_dummy = (Integer)
            $context.getValue("/toto/dummy",'/');
         assertSame($jscheduler_dummy,$toto_dummy);
	/**
         Scheduler $jscheduler = (Scheduler)
            $context.getValue("/JScheduler/.instance",'/');
         Scheduler $toto_scheduler = (Scheduler)
            $context.getValue("/toto/scheduler",'/'); 

	    */
       //  assert("Schedulers should be different.",! $jscheduler.equals($toto_scheduler));
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      }
   }
   
   public void testImportXML2() {
      try {

         Context $initial_context = Kernel.newConfiguration(OpenXMLDecoderTest.class);
         
         File2Configuration $decoder = (File2Configuration) 
            $initial_context.getValue("jonathan/decoders/OpenXMLDecoder/instance",'/');
         ClassPathFinder $finder = (ClassPathFinder) $initial_context.getValue("classpath_finder",(char) 0);
         
         JConfigurationFactory $context_factory = new JConfigurationFactory();

         Context $context = $context_factory.newContext(null,$finder);

         $decoder.importElement($context,
                                "org/objectweb/jonathan/libs/kernel/decoders/BuildersObjects.xml",
                                $context_factory,
                                $finder);

         Object $component = $context.getValue("substate1",(char) 0);
         assert("substate1 is not a context.", $component instanceof Context);
         $component = $context.getValue("substate2",(char) 0);
         assert("substate2 is not a context.", $component instanceof Context);

         Enumeration $enum = $context.getElements();
         
         Element $element = null;
         $element = (Element) $enum.nextElement();
         assertEquals("Bad element found.","substate1",$element.getName());
         $element = (Element) $enum.nextElement();
         assertEquals("Bad element found.","substate2",$element.getName());
         
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      }
   }
   
   public void testImportXMLWithInclude() {
      try {
         
         Context $initial_context = Kernel.newConfiguration(OpenXMLDecoderTest.class);

         ClassPathFinder $finder = (ClassPathFinder) $initial_context.getValue("classpath_finder",(char) 0);
         File2Configuration $decoder = (File2Configuration) 
            $initial_context.getValue("jonathan/decoders/OpenXMLDecoder/instance",'/');

         
         JConfigurationFactory $context_factory = new JConfigurationFactory();

         Context $context = $context_factory.newContext(null,$finder);

         $decoder.importElement($context,
                                "org/objectweb/jonathan/libs/kernel/decoders/Include.xml",
                                $context_factory,
                                $finder);


         Object $component = $context.getValue("builders/substate1",'/');
         assert("builders/substate1 is not a context.", $component instanceof Context);
         $component = $context.getValue("builders/substate2",'/');
         assert("builders/substate2 is not a context.", $component instanceof Context);
         $component = $context.getValue("subelem",(char) 0);
         assert("subelem is not a context.", $component instanceof Context);

         Enumeration $enum = ((Context) $component).getElements();
         
         Element $element = null;
         $element = (Element) $enum.nextElement();
         assertEquals("Bad element found.","substate1",$element.getName());
         $element = (Element) $enum.nextElement();
         assertEquals("Bad element found.","substate2",$element.getName());
      } catch (Exception e) {
         e.printStackTrace();
         fail("Unexpected exception: " + e); 
      }
   }

}

