/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 *
 * -------------------------------------------------------------------
 * $ID$
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.apis.kernel;

/** 
 * <code>MultiFactory</code> complements the  {@link Factory Factory} type
 * with the ability for some factories to provide different 
 * implementation alternatives.
 */
public interface MultiFactory {

   /** 
    * Returns an object created using information contained in the provided 
    * {@link Context context}.
    * @param context a {@link Context Context} object;
    * @param alternative the implementation alternative to use.
    * @param instanciable the method's caller.
    * @return an object created by the factory.
    * @exception JonathanException if something goes wrong.
    */
   Object newObject(Context context,int alternative, Instanciable instanciable) throws JonathanException;
}
