/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 * 
 * --------------------------------------------------------------------------
 * $Id: JAlias.java,v 1.1.1.1 2002/08/06 13:13:51 bmi Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.Element;
import org.objectweb.jonathan.apis.kernel.Name;
import org.objectweb.jonathan.apis.kernel.JonathanException;

import org.objectweb.jonathan.libs.kernel.TreeElement;
import org.objectweb.jonathan.libs.kernel.Forkable;
import org.objectweb.jonathan.libs.helpers.NameHelpers;

/**
 * The alias runtime component implementation.
 */
public class JAlias implements Forkable {
      
   boolean initialized;
      
   /** the referenced element */
   protected Element alias;
      
   protected Name alias_name;

   protected Element reference;
      
   public JAlias(String _alias_name,char separator) {
      this(NameHelpers.newName(_alias_name,separator));
   }
      
   public JAlias(Name _alias_name) {
      alias_name = _alias_name;
      initialized = false;
      reference = null;
   }
      
   public Class getType(){
      initialize();
      return alias != null ? alias.getType() : null;
   }
         
   public Object getValue() {
      initialize();
      return alias != null ? alias.getValue() : Context.NO_VALUE;
   }
      
   public int getIntValue() {
      initialize();
      return alias != null ? alias.getIntValue() : Integer.MAX_VALUE;
   }

   public Object getFactoryValue() throws JonathanException{
      initialize();
      if (alias != null){
         Component $comp = alias.getComponent();
         if (($comp != null) && ($comp != Context.NO_VALUE)){
            return $comp.getFactoryValue();
         }
      }
      return Context.NO_VALUE;
   }

   public Element getReference() {
      return reference;
   }
      
   public Element setReference(Element _reference) {
      Element $former = reference;
      reference = _reference;
      return $former;
   }
      
   public String toString() {
      if (initialized) {
         return "JAlias[" + alias_name + "]";
      } else {
         return "JAlias[" + alias_name + " " + alias + "]";
      }            
   }

   /**
    * JAlias specific initialization code. Retrieves the aliased
    * {@link Element} (but don't instantiate its value!). Used
    * used internally by {@link org.objectweb.jonathan.libs.kernel.JComponent}.
    */      
   final protected void initialize() {
      if (! initialized) {
         initialized = true;
         if (reference != null) {
            if (alias_name != null) {
               String alias_id = alias_name.getId();
               if (alias_id.length() == 0) {
                  Name my_name = null;
                  if (reference instanceof TreeElement) {
                     my_name = NameHelpers.getPrefix(((TreeElement) reference).getAbsoluteName());
                     String my_id = my_name.getId();
                     while (my_id.equals(alias_id)) {
                        alias_name = alias_name.getSubName();
                        my_name = my_name.getSubName();
                        if (alias_name == null) {
                        break;
                        } else {
                           alias_id = alias_name.getId();
                        }
                        if (my_name == null) {
                           break;
                        } else {
                           my_id = my_name.getId();
                        }
                     }
                  }
                  if (alias_name == my_name) {
                     alias_name = NameHelpers.newName(".",null);
                  } else {
                     while (my_name != null) {
                        alias_name = NameHelpers.newName("..",alias_name);
                        my_name = my_name.getSubName();
                     }
                  }
               }
            }
            alias = reference.getContainer().getElement(alias_name);
         } 
         if (alias == null) {
            System.err.println("WARNING: alias of name " + alias_name + " not found.");
         } else {
            initialized = false;
         }
      }
   }
   public Component internalFork() {
      Component result = new JAlias(alias_name);
      result.setReference(reference);
      return result;
   }

   public Component fork() {
      initialize();
      if (alias != null){
         Component $comp = alias.getComponent();
         if ($comp != null) {
            return $comp.fork();
         }
      }
      return internalFork();
   }   

   public Element getElement(Name name) {
      initialize();
      if (alias != null) {
         Component $target = alias.getComponent();
         if ($target != null) {
            return $target.getElement(name);
         }
      }
      return null;
   }
   
   final public Component getComponent(Name _name) {
      Element $element = getElement(_name);
      if ($element != null) {
         return $element.getComponent();
      } else {
         return null;
      }
   }
      
   final public Object getValue(Name _name) {
      Element element = getElement(_name);
      if (element != null) {
         return element.getValue();
      } else {
         return Context.NO_VALUE;
      }
   }
}
