/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 *
 * -------------------------------------------------------------------
 * $ID$
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.JonathanException;

import nanoxml.XMLElement;

/**
 * Alias specific XML to runtime component decoder.
 * Used by the {@link OpenXMLDecoder} as one of the built-in
 * component decoders provided it provides to its decoding
 * dispatcher.
 */
public class XMLAliasDecoder implements XMLElementDecoder {
   
   /**
    * Converts a XML <code>ALIAS</code> element to its
    * corresponding alias runtime component.
    * @param _xml_element <code>ALIAS</code> tag to be decoded
    * @param _element_name the alias's element name
    * @param _configuration where the alias will be created
    * @param return the corresponding decoded runtime alias
    * component
    */
   public Component decode(XMLElement _xml_element, String _element_name, Context _configuration,
                           XMLElementDecoderDispatcher _dispatcher) 
      throws JonathanException {
      
      String $current_tag = _xml_element.getName();

      if ($current_tag.equals("ALIAS")){
         String $alias = _xml_element.getStringAttribute("NAME");
         return new JAlias($alias,'/');
      } else {
         return _dispatcher.error("XMLAliasDecoder error: Wrong tag " + $current_tag);
      }
   }
}
