/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Priscille Durville
 *
 * -------------------------------------------------------------------
 * $Id: XMLAssemblageDecoder.java,v 1.2 2002/09/30 13:29:43 dumantbr Exp $
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.JonathanException;

import org.objectweb.jonathan.libs.kernel.decoders.JAlias;
import org.objectweb.jonathan.libs.kernel.decoders.XMLElementDecoder;
import org.objectweb.jonathan.libs.kernel.decoders.XMLElementDecoderDispatcher;

import nanoxml.XMLElement;

import java.util.Enumeration;
import java.util.List;
import java.util.LinkedList;

/**
 * Assemblage specific XML to runtime component decoder.
 * Used by the {@link OpenXMLDecoder} as one of the built-in
 * component decoders provided it provides to its decoding
 * dispatcher.
 */
public class XMLAssemblageDecoder implements XMLElementDecoder {
   
   /**
    * Converts a XML <code>ASSEMBLAGE</code> element to its
    * corresponding assemblage runtime component.
    * @param _xml_element <code>ASSEMBLAGE</code> tag to be decoded
    * @param _element_name the assemblage's element name
    * @param _configuration where the assemblage will be created
    * @param return the corresponding decoded runtime assemblage
    * component
    */
   public Component decode(XMLElement _xml_element, String _element_name, Context _configuration,
                           XMLElementDecoderDispatcher _dispatcher) 
      throws JonathanException {
      
      String $current_tag = _xml_element.getName();

      if ($current_tag.equals("ASSEMBLAGE")){

         Component $factory = null;
         Component $configuration = null;
         List $alternatives_list = new LinkedList();

         Enumeration $members = _xml_element.enumerateChildren();

         while ($members.hasMoreElements()) {
            XMLElement $argument = (XMLElement) $members.nextElement();
            $current_tag = $argument.getName();
            if ($current_tag.equals("MY_FACTORY")) {
               if ($factory != null) {
                  return _dispatcher.error("Duplicate factory declaration at line " + 
                                           _xml_element.getLineNr());
               } else {
                  Enumeration $factory_members = $argument.enumerateChildren();
                  while ($factory_members.hasMoreElements()) {
                     XMLElement $factory_member = (XMLElement) $factory_members.nextElement();
                     if ($factory != null) {
                        return _dispatcher.error("More than one component in MY_FACTORY at line " + 
                                                 $factory_member.getLineNr());
                     } else {
                        $factory = _dispatcher.decode($factory_member,null,null);
                     }
                  }
               }
            } else if ($current_tag.equals("MY_CONFIGURATION")) {
               if ($configuration != null) {
                  return _dispatcher.error("Duplicate configuration declaration at line " + 
                                           _xml_element.getLineNr());
               } else {
                  Enumeration $configuration_members = $argument.enumerateChildren();
                  while ($configuration_members.hasMoreElements()) {
                     XMLElement $configuration_member = (XMLElement) $configuration_members.nextElement();
                     if ($configuration != null) {
                        return _dispatcher.error("More than one component in MY_CONFIGURATION at line " + 
                                                 $configuration_member.getLineNr());
                     } else {
                        $configuration = _dispatcher.decode($configuration_member,null,null);
                     }
                  }
               }
            } else if ($current_tag.equals("ALTERNATIVE")){
               List $arguments_list = new LinkedList();
               List $setters_list = new LinkedList();
               Enumeration $arguments = $argument.enumerateChildren();
               while ($arguments.hasMoreElements()) {
                  XMLElement $arg = (XMLElement) $arguments.nextElement();
                  $current_tag = $arg.getName();
                  if ($current_tag.equals("ARGUMENT")) {
                     $arguments_list.add(new JArgument($arg.getStringAttribute("NAME"),
                                                       $arg.getStringAttribute("TYPE"))); 
                  } else if ($current_tag.equals("SETTER")){
                     LinkedList $setter_arg_list = new LinkedList();
                     Enumeration $setter_arguments = $arg.enumerateChildren();
                     while ($setter_arguments.hasMoreElements()) {
                        XMLElement $current = (XMLElement)$setter_arguments.nextElement();
                        if ($current.getName().equals("ARGUMENT")){
                           $setter_arg_list.add(new JArgument($current.getStringAttribute("NAME"),
                                                              $current.getStringAttribute("TYPE")));
                        }
                     }
                     $setters_list.add(new JSetter($arg.getStringAttribute("NAME"),$setter_arg_list));
                  } else {
                     return _dispatcher.error("Bad tag: " + $current_tag +". ARGUMENT expected.");
                  }
               }
               String $alternative_id_property = $argument.getStringAttribute("ID");
               int $alternative_id = 0;
               if ($alternative_id_property != null) {
                  try {
                     $alternative_id = Integer.parseInt($alternative_id_property);
                  } catch (NumberFormatException e) {
                     throw new JonathanException("Tag ID of alternative at line " + $argument.getLineNr() +
                                                 " must be a positive integer.");
                  }
               } else {
                  throw new JonathanException("Tag ID of alternative at line "+ $argument.getLineNr() +
                                              " required.");
               }
               $alternatives_list.add(new JAlternative($argument.getStringAttribute("INIT"),
                                                       $argument.getStringAttribute("CLASS"),
                                                       $arguments_list,
                                                       $setters_list,
                                                       $alternative_id));
            } else if ($current_tag.equals("DOCUMENTATION")){
               //no exception.
            }
         }
            
         String $factory_attribute = _xml_element.getStringAttribute("FACTORY");
         if ($factory_attribute == null && $factory == null) {
            return _dispatcher.error("No factory defined for assemblage at line: " + 
                                     _xml_element.getLineNr());
         } else if ($factory_attribute != null && $factory != null) {            
            return _dispatcher.error("Duplicate configuration declaration at line " + 
                                     _xml_element.getLineNr());
         } else if ($factory_attribute != null) {
            $factory = new JAlias("../" + $factory_attribute,'/');
         }
         
         String $configuration_attribute = _xml_element.getStringAttribute("CONFIGURATION");
         if ($configuration_attribute == null && $configuration == null) {
            //return _dispatcher.error("No configuration defined for assemblage at line: " + 
            //                         _xml_element.getLineNr());	
            $configuration = _dispatcher.getContextFactory().newContext();
         } else if ($configuration_attribute != null && $configuration != null) {            
            return _dispatcher.error("Duplicate configuration declaration at line " + 
                                     _xml_element.getLineNr());
         } else if ($configuration_attribute != null) {
            $configuration = new JAlias("../" + $configuration_attribute,'/');
         }

         String $choice_spec = _xml_element.getStringAttribute("ALTERNATIVE");
         int $choice = -1;
         if ($choice_spec != null) {
            try {
               $choice = Integer.parseInt($choice_spec);
               return new JAssemblage($factory, $configuration, $choice, $alternatives_list);
            } catch (NumberFormatException e) {
               return _dispatcher.error("Attribute ALTERNATIVE on line " + _xml_element.getLineNr() + 
                                        " must be parsable as an integer.");
            }
         } else {
            return new JAssemblage($factory,$configuration,$alternatives_list);
         }
      } else {
         return _dispatcher.error("XMLAssemblageDecoder error: Wrong tag " + $current_tag);
      }
   }
}
