/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 *
 * -------------------------------------------------------------------
 * $ID$
 * -------------------------------------------------------------------
 * 
 */

package org.objectweb.jonathan.libs.kernel.decoders;

import org.objectweb.jonathan.apis.kernel.Context;
import org.objectweb.jonathan.apis.kernel.Component;
import org.objectweb.jonathan.apis.kernel.JonathanException;

import org.objectweb.jonathan.libs.kernel.decoders.XMLElementDecoder;
import org.objectweb.jonathan.libs.kernel.decoders.XMLElementDecoderDispatcher;

import nanoxml.XMLElement;

import java.util.Enumeration;
import java.util.List;
import java.util.LinkedList;

/**
 * Imports a runtime implicit factory component from its XML description.
 * See the XML DTD file. Allow an XMLOpenDecoder to decode
 * an XML configuration file containing valid implicit factory tags.
 * It must therefore be provided in the Context parameter of the
 * <code>OpenXMLDecoder</code>as the value of an element named
 * "IMPLICIT_FACTORY".
 * @see org.objectweb.jonathan.libs.kernel.decoders.JImplicitFactory
 * @see org.objectweb.jonathan.libs.kernel.decoders.OpenXMLDecoder
 */
public class XMLImplicitFactoryDecoder implements XMLElementDecoder {
   
   /**
    * returns a new runtime ImplicitFactory component from
    * its NanoXML element description.
    * @param _xml_element its NanoXML description
    * @param _element_name its element's name
    * @param _configuration unused here
    * @return the corresponding runtime implicit factory component
    */
   public Component decode(XMLElement _xml_element, String _element_name, Context _configuration,
                           XMLElementDecoderDispatcher _dispatcher) 
      throws JonathanException {
      
      String $current_tag = _xml_element.getName();

      if ($current_tag.equals("IMPLICIT_FACTORY")) {
         Enumeration $alternatives = _xml_element.enumerateChildren();
         List $alternatives_list = new LinkedList();
         while ($alternatives.hasMoreElements()) {
            XMLElement $alternative = (XMLElement) $alternatives.nextElement();
            $current_tag = $alternative.getName();
            if ($current_tag.equals("ALTERNATIVE")) {
               List $arguments_list = new LinkedList();
               List $setters_list = new LinkedList();
               Enumeration $arguments = $alternative.enumerateChildren();
               while ($arguments.hasMoreElements()) {
                  XMLElement $argument = (XMLElement) $arguments.nextElement();
                  $current_tag = $argument.getName();
                  if ($current_tag.equals("ARGUMENT")) {
                     $arguments_list.add(new JArgument($argument.getStringAttribute("NAME"),
                                                      $argument.getStringAttribute("TYPE"))); 
                  } else if ($current_tag.equals("SETTER")){
                     LinkedList $setter_arg_list = new LinkedList();
                     Enumeration $setter_arguments = $argument.enumerateChildren();
                     while ($setter_arguments.hasMoreElements()) {
                        XMLElement $current = (XMLElement)$setter_arguments.nextElement();
                        if ($current.getName().equals("ARGUMENT")){
                           $setter_arg_list.add(new JArgument($current.getStringAttribute("NAME"),
                                                              $current.getStringAttribute("TYPE")));
                        }
                     }
                     $setters_list.add(new JSetter($argument.getStringAttribute("NAME"),$setter_arg_list));
                  } else {
                     return _dispatcher.error("Bad tag: " + $current_tag +". ARGUMENT expected.");
                  }
               }
               String $alternative_id_property = $alternative.getStringAttribute("ID");
               int $alternative_id = 0;
               if ($alternative_id_property != null) {
                  try {
                     $alternative_id = Integer.parseInt($alternative_id_property);
                  } catch (NumberFormatException e) {
                     throw new JonathanException("Tag ID of alternative at line " + $alternative.getLineNr() +
                                                 " must be a positive integer.");
                  }
               } else {
                  throw new JonathanException("Tag ID of alternative at line " + $alternative.getLineNr() +
                                              " required.");
               }
               $alternatives_list.add(new JAlternative($alternative.getStringAttribute("CLASS"),
                                                       $arguments_list,
                                                       $setters_list,
                                                       $alternative_id));
            } else if ($current_tag.equals("DOCUMENTATION")){
               //no exception.
            } else {
               return _dispatcher.error("Bad tag: " + $current_tag +". ALTERNATIVE expected.");
            }
         }
         return new JImplicitFactory($alternatives_list);

      } else {
         return _dispatcher.error("XMLImplicitFactoryDecoder error: Wrong tag " + $current_tag);
      }
   }
}
