/***
 * Jonathan: an Open Distributed Processing Environment 
 * Copyright (C) 2001 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: jonathan@objectweb.org
 *
 * Author: Bruno Dumant
 * 
 */

package org.objectweb.jonathan.tools.compilers;

import org.objectweb.jonathan.tools.Kcf2java;
import nanoxml.XMLElement;
import nanoxml.XMLParseException;
import org.objectweb.jonathan.tools.compilers.TagCompiler;

import java.io.IOException;
import java.io.Writer;

import java.util.Enumeration;

/**
 * Implements the Sequence XML to java compilation.
 * Used by the XML to java translator: {@link Kcf2java}.
 * This is not an Kcf2java built-in tag compiler.
 * It must therefore be provided in the Kcf2java property
 * file (which is an Kcf2java command line argument) in
 * order to allow Kcf2java to compile XML configuration
 * files with containing valid implicit factory tags.
 */
public class SequenceCompiler implements TagCompiler {

   static int nb = 0;
   static int alternatives_list_number = 0;
   static int arguments_list_number = 0;

   /**
    * Writes the java code translation of the given sequence
    * XML description (as a {@link XMLElement}).
    * @param _current_component the sequence component
    * XML description
    * @param _element_name its element's name
    * @param _configuration_var_name unused
    * @param _writer used to write the java translation code
    * @param _configuration_name unused
    * @return the newly created sequence java component's
    * variable name
    */
   public String  writeComponent(XMLElement _current_component,
                                 String _element_name,
                                 String _configuration_var_name,
                                 Writer _writer,
                                 String _configuration_name)
      throws IOException {
      
      String $current_tag = _current_component.getName();
      
      if ($current_tag.equals("SEQUENCE")) {
         String $var_name = "SEQUENCE" + (nb++);

         String $elements_type = _current_component.getStringAttribute("ELEMENTS_TYPE");
         if ($elements_type == null || $elements_type.equals("")) {
            throw new IOException("invalid elements_type at line " +
                                  _current_component.getLineNr());
         }

         if ("String".equals($elements_type)) {
            $elements_type = "java.lang.String";
         }

         String $elements_list_var = "$sequence_elements_list" + (nb);
         _writer.write("        java.util.Vector " + $elements_list_var + " = new java.util.Vector();\n");

         Enumeration $elements = _current_component.enumerateChildren();

         int $count = 0;
         while ($elements.hasMoreElements()) {
            XMLElement $ELEM_elem = (XMLElement)$elements.nextElement();

            if ($ELEM_elem.getName().equals("DOCUMENTATION")) {
            } else if (! ($ELEM_elem.getName().equals("ELEM"))) {
               throw new IOException("Sequence elements must be enclosed in ELEM tag at line " + _current_component.getLineNr());
            } else {
               XMLElement $each = (XMLElement)$ELEM_elem.enumerateChildren().nextElement();
               String $each_element_name = Kcf2java.writeComponent($each,Integer.toString($count),
                                                                   null, _writer,
                                                                   _configuration_name + "/" + _element_name);
               
               _writer.write("        "+ $elements_list_var + ".add(" + $each_element_name + ");\n");
            }
         }

            _writer.write("         Component " + $var_name +
                       " = new org.objectweb.jonathan.libs.kernel.decoders.JSequence(\"" + 
                       $elements_type + "\", "+ $elements_list_var + ".elements());\n");
         return $var_name;
      } else {
         throw new IOException("TAG error at line " + _current_component.getLineNr() + ":\n" +
                               "SEQUENCE excepted instead of " + $current_tag);
      }      
   }
}
